﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Benchmarks {
  public static class Util {
    public static ISymbolicExpressionTree[] CreateRandomTrees(IRandom random, Dataset dataset, ISymbolicExpressionGrammar grammar,
      int popSize, int minSize, int maxSize,
      int maxFunctionDefinitions, int maxFunctionArguments) {
      foreach (var variableSymbol in grammar.Symbols.OfType<HeuristicLab.Problems.DataAnalysis.Symbolic.Variable>()) {
        variableSymbol.VariableNames = dataset.VariableNames.Skip(1);
      }
      ISymbolicExpressionTree[] randomTrees = new ISymbolicExpressionTree[popSize];
      for (int i = 0; i < randomTrees.Length; i++) {
        randomTrees[i] = ProbabilisticTreeCreator.Create(random, grammar, maxSize, 10);
      }
      return randomTrees;
    }

    public static Dataset CreateRandomDataset(IRandom random, int rows, int columns) {
      double[,] data = new double[rows, columns];
      for (int i = 0; i < rows; i++) {
        for (int j = 0; j < columns; j++) {
          data[i, j] = random.NextDouble() * 2.0 - 1.0;
        }
      }
      IEnumerable<string> variableNames = new string[] { "y" }.Concat(Enumerable.Range(0, columns - 1).Select(x => "x" + x.ToString()));
      Dataset ds = new Dataset(variableNames, data);
      return ds;
    }
  }
}
